@@ -142,13 +142,7 @@ static bool process_get_lwp_regs(struct ps_prochandle* ph, pid_t pid, struct use
 #define PTRACE_GETREGS_REQ PT_GETREGS
 #endif
 
-#ifdef PTRACE_GETREGS_REQ
- if (ptrace_getregs(PTRACE_GETREGS_REQ, pid, user, NULL) < 0) {
-   print_debug("ptrace(PTRACE_GETREGS, ...) failed for lwp %d\n", pid);
-   return false;
- }
- return true;
-#elif defined(PTRACE_GETREGSET)
+#if defined(PTRACE_GETREGSET)
  struct iovec iov;
  iov.iov_base = user;
  iov.iov_len = sizeof(*user);
@@ -157,6 +151,12 @@ static bool process_get_lwp_regs(struct ps_prochandle* ph, pid_t pid, struct use
    return false;
  }
  return true;
+#elif defined(PTRACE_GETREGS_REQ)
+ if (ptrace_getregs(PTRACE_GETREGS_REQ, pid, user, NULL) < 0) {
+   print_debug("ptrace(PTRACE_GETREGS, ...) failed for lwp %d\n", pid);
+   return false;
+ }
+ return true;
 #else
  print_debug("ptrace(PTRACE_GETREGS, ...) not supported\n");
  return false;
@@ -282,8 +282,17 @@ static attach_state_t ptrace_attach(pid_t pid, char* err_buf, size_t err_buf_len
         return ATTACH_THREAD_DEAD;
       }
     }
+
+    // strerror_r() API function is not compatible in different implementations:
+    // GNU-specific:  char *strerror_r(int errnum, char *buf, size_t buflen);
+    // XSI-compliant: int   strerror_r(int errnum, char *buf, size_t buflen);
     char buf[200];
+#if defined(__GLIBC__) && defined(_GNU_SOURCE)
     char* msg = strerror_r(errno, buf, sizeof(buf));
+#else
+    int rc = strerror_r(errno, buf, sizeof(buf));
+    char* msg = (rc == 0) ? (char*)buf : "Unknown";
+#endif
     snprintf(err_buf, err_buf_len, "ptrace(PTRACE_ATTACH, ..) failed for %d: %s", pid, msg);
     print_error("%s\n", err_buf);
     return ATTACH_FAIL;
