@@ -20,6 +20,8 @@
  * or visit www.oracle.com if you need additional information or have any
  * questions.
  */
+
+
 package org.graalvm.compiler.replacements;
 
 import static jdk.vm.ci.code.MemoryBarriers.JMM_POST_VOLATILE_READ;
@@ -41,6 +43,7 @@ import java.util.List;
 import org.graalvm.compiler.api.directives.GraalDirectives;
 import org.graalvm.compiler.api.replacements.Snippet;
 import org.graalvm.compiler.api.replacements.SnippetReflectionProvider;
+import org.graalvm.compiler.core.common.LIRKind;
 import org.graalvm.compiler.core.common.spi.ForeignCallDescriptor;
 import org.graalvm.compiler.core.common.spi.ForeignCallsProvider;
 import org.graalvm.compiler.core.common.type.IntegerStamp;
@@ -48,6 +51,7 @@ import org.graalvm.compiler.core.common.type.ObjectStamp;
 import org.graalvm.compiler.core.common.type.Stamp;
 import org.graalvm.compiler.core.common.type.StampFactory;
 import org.graalvm.compiler.core.common.type.TypeReference;
+import org.graalvm.compiler.debug.DebugCloseable;
 import org.graalvm.compiler.debug.DebugHandlersFactory;
 import org.graalvm.compiler.debug.GraalError;
 import org.graalvm.compiler.graph.Node;
@@ -83,6 +87,7 @@ import org.graalvm.compiler.nodes.extended.GuardedUnsafeLoadNode;
 import org.graalvm.compiler.nodes.extended.GuardingNode;
 import org.graalvm.compiler.nodes.extended.JavaReadNode;
 import org.graalvm.compiler.nodes.extended.JavaWriteNode;
+import org.graalvm.compiler.nodes.extended.LoadArrayComponentHubNode;
 import org.graalvm.compiler.nodes.extended.LoadHubNode;
 import org.graalvm.compiler.nodes.extended.MembarNode;
 import org.graalvm.compiler.nodes.extended.RawLoadNode;
@@ -108,7 +113,9 @@ import org.graalvm.compiler.nodes.java.NewInstanceNode;
 import org.graalvm.compiler.nodes.java.RawMonitorEnterNode;
 import org.graalvm.compiler.nodes.java.StoreFieldNode;
 import org.graalvm.compiler.nodes.java.StoreIndexedNode;
+import org.graalvm.compiler.nodes.java.UnsafeCompareAndExchangeNode;
 import org.graalvm.compiler.nodes.java.UnsafeCompareAndSwapNode;
+import org.graalvm.compiler.nodes.java.ValueCompareAndSwapNode;
 import org.graalvm.compiler.nodes.memory.HeapAccess.BarrierType;
 import org.graalvm.compiler.nodes.memory.ReadNode;
 import org.graalvm.compiler.nodes.memory.WriteNode;
@@ -131,19 +138,19 @@ import org.graalvm.compiler.replacements.nodes.BinaryMathIntrinsicNode;
 import org.graalvm.compiler.replacements.nodes.BinaryMathIntrinsicNode.BinaryOperation;
 import org.graalvm.compiler.replacements.nodes.UnaryMathIntrinsicNode;
 import org.graalvm.compiler.replacements.nodes.UnaryMathIntrinsicNode.UnaryOperation;
-import org.graalvm.word.LocationIdentity;
+import jdk.internal.vm.compiler.word.LocationIdentity;
 
 import jdk.vm.ci.code.CodeUtil;
 import jdk.vm.ci.code.MemoryBarriers;
 import jdk.vm.ci.code.TargetDescription;
 import jdk.vm.ci.meta.DeoptimizationAction;
 import jdk.vm.ci.meta.DeoptimizationReason;
-import jdk.vm.ci.meta.JavaConstant;
 import jdk.vm.ci.meta.JavaKind;
 import jdk.vm.ci.meta.MetaAccessProvider;
 import jdk.vm.ci.meta.ResolvedJavaField;
 import jdk.vm.ci.meta.ResolvedJavaMethod;
 import jdk.vm.ci.meta.ResolvedJavaType;
+import jdk.vm.ci.meta.SpeculationLog;
 
 /**
  * VM-independent lowerings for standard Java nodes. VM-specific methods are abstract and must be
@@ -155,6 +162,7 @@ public abstract class DefaultJavaLoweringProvider implements LoweringProvider {
     protected final ForeignCallsProvider foreignCalls;
     protected final TargetDescription target;
     private final boolean useCompressedOops;
+    private final ResolvedJavaType objectArrayType;
 
     private BoxingSnippets.Templates boxingSnippets;
     private ConstantStringIndexOfSnippets.Templates indexOfSnippets;
@@ -164,6 +172,7 @@ public abstract class DefaultJavaLoweringProvider implements LoweringProvider {
         this.foreignCalls = foreignCalls;
         this.target = target;
         this.useCompressedOops = useCompressedOops;
+        this.objectArrayType = metaAccess.lookupJavaType(Object[].class);
     }
 
     public void initialize(OptionValues options, Iterable<DebugHandlersFactory> factories, SnippetCounter.Group.Factory factory, Providers providers, SnippetReflectionProvider snippetReflection) {
@@ -177,57 +186,64 @@ public abstract class DefaultJavaLoweringProvider implements LoweringProvider {
     }
 
     @Override
+    @SuppressWarnings("try")
     public void lower(Node n, LoweringTool tool) {
         assert n instanceof Lowerable;
         StructuredGraph graph = (StructuredGraph) n.graph();
-        if (n instanceof LoadFieldNode) {
-            lowerLoadFieldNode((LoadFieldNode) n, tool);
-        } else if (n instanceof StoreFieldNode) {
-            lowerStoreFieldNode((StoreFieldNode) n, tool);
-        } else if (n instanceof LoadIndexedNode) {
-            lowerLoadIndexedNode((LoadIndexedNode) n, tool);
-        } else if (n instanceof StoreIndexedNode) {
-            lowerStoreIndexedNode((StoreIndexedNode) n, tool);
-        } else if (n instanceof ArrayLengthNode) {
-            lowerArrayLengthNode((ArrayLengthNode) n, tool);
-        } else if (n instanceof LoadHubNode) {
-            lowerLoadHubNode((LoadHubNode) n, tool);
-        } else if (n instanceof MonitorEnterNode) {
-            lowerMonitorEnterNode((MonitorEnterNode) n, tool, graph);
-        } else if (n instanceof UnsafeCompareAndSwapNode) {
-            lowerCompareAndSwapNode((UnsafeCompareAndSwapNode) n);
-        } else if (n instanceof AtomicReadAndWriteNode) {
-            lowerAtomicReadAndWriteNode((AtomicReadAndWriteNode) n);
-        } else if (n instanceof RawLoadNode) {
-            lowerUnsafeLoadNode((RawLoadNode) n, tool);
-        } else if (n instanceof UnsafeMemoryLoadNode) {
-            lowerUnsafeMemoryLoadNode((UnsafeMemoryLoadNode) n);
-        } else if (n instanceof RawStoreNode) {
-            lowerUnsafeStoreNode((RawStoreNode) n);
-        } else if (n instanceof UnsafeMemoryStoreNode) {
-            lowerUnsafeMemoryStoreNode((UnsafeMemoryStoreNode) n);
-        } else if (n instanceof JavaReadNode) {
-            lowerJavaReadNode((JavaReadNode) n);
-        } else if (n instanceof JavaWriteNode) {
-            lowerJavaWriteNode((JavaWriteNode) n);
-        } else if (n instanceof CommitAllocationNode) {
-            lowerCommitAllocationNode((CommitAllocationNode) n, tool);
-        } else if (n instanceof BoxNode) {
-            boxingSnippets.lower((BoxNode) n, tool);
-        } else if (n instanceof UnboxNode) {
-            boxingSnippets.lower((UnboxNode) n, tool);
-        } else if (n instanceof VerifyHeapNode) {
-            lowerVerifyHeap((VerifyHeapNode) n);
-        } else if (n instanceof UnaryMathIntrinsicNode) {
-            lowerUnaryMath((UnaryMathIntrinsicNode) n, tool);
-        } else if (n instanceof BinaryMathIntrinsicNode) {
-            lowerBinaryMath((BinaryMathIntrinsicNode) n, tool);
-        } else if (n instanceof StringIndexOfNode) {
-            lowerIndexOf((StringIndexOfNode) n);
-        } else if (n instanceof UnpackEndianHalfNode) {
-            lowerSecondHalf((UnpackEndianHalfNode) n);
-        } else {
-            throw GraalError.shouldNotReachHere("Node implementing Lowerable not handled: " + n);
+        try (DebugCloseable context = n.withNodeSourcePosition()) {
+            if (n instanceof LoadFieldNode) {
+                lowerLoadFieldNode((LoadFieldNode) n, tool);
+            } else if (n instanceof StoreFieldNode) {
+                lowerStoreFieldNode((StoreFieldNode) n, tool);
+            } else if (n instanceof LoadIndexedNode) {
+                lowerLoadIndexedNode((LoadIndexedNode) n, tool);
+            } else if (n instanceof StoreIndexedNode) {
+                lowerStoreIndexedNode((StoreIndexedNode) n, tool);
+            } else if (n instanceof ArrayLengthNode) {
+                lowerArrayLengthNode((ArrayLengthNode) n, tool);
+            } else if (n instanceof LoadHubNode) {
+                lowerLoadHubNode((LoadHubNode) n, tool);
+            } else if (n instanceof LoadArrayComponentHubNode) {
+                lowerLoadArrayComponentHubNode((LoadArrayComponentHubNode) n);
+            } else if (n instanceof MonitorEnterNode) {
+                lowerMonitorEnterNode((MonitorEnterNode) n, tool, graph);
+            } else if (n instanceof UnsafeCompareAndSwapNode) {
+                lowerCompareAndSwapNode((UnsafeCompareAndSwapNode) n);
+            } else if (n instanceof UnsafeCompareAndExchangeNode) {
+                lowerCompareAndExchangeNode((UnsafeCompareAndExchangeNode) n);
+            } else if (n instanceof AtomicReadAndWriteNode) {
+                lowerAtomicReadAndWriteNode((AtomicReadAndWriteNode) n);
+            } else if (n instanceof RawLoadNode) {
+                lowerUnsafeLoadNode((RawLoadNode) n, tool);
+            } else if (n instanceof UnsafeMemoryLoadNode) {
+                lowerUnsafeMemoryLoadNode((UnsafeMemoryLoadNode) n);
+            } else if (n instanceof RawStoreNode) {
+                lowerUnsafeStoreNode((RawStoreNode) n);
+            } else if (n instanceof UnsafeMemoryStoreNode) {
+                lowerUnsafeMemoryStoreNode((UnsafeMemoryStoreNode) n);
+            } else if (n instanceof JavaReadNode) {
+                lowerJavaReadNode((JavaReadNode) n);
+            } else if (n instanceof JavaWriteNode) {
+                lowerJavaWriteNode((JavaWriteNode) n);
+            } else if (n instanceof CommitAllocationNode) {
+                lowerCommitAllocationNode((CommitAllocationNode) n, tool);
+            } else if (n instanceof BoxNode) {
+                boxingSnippets.lower((BoxNode) n, tool);
+            } else if (n instanceof UnboxNode) {
+                boxingSnippets.lower((UnboxNode) n, tool);
+            } else if (n instanceof VerifyHeapNode) {
+                lowerVerifyHeap((VerifyHeapNode) n);
+            } else if (n instanceof UnaryMathIntrinsicNode) {
+                lowerUnaryMath((UnaryMathIntrinsicNode) n, tool);
+            } else if (n instanceof BinaryMathIntrinsicNode) {
+                lowerBinaryMath((BinaryMathIntrinsicNode) n, tool);
+            } else if (n instanceof StringIndexOfNode) {
+                lowerIndexOf((StringIndexOfNode) n);
+            } else if (n instanceof UnpackEndianHalfNode) {
+                lowerSecondHalf((UnpackEndianHalfNode) n);
+            } else {
+                throw GraalError.shouldNotReachHere("Node implementing Lowerable not handled: " + n);
+            }
         }
     }
 
@@ -444,7 +460,7 @@ public abstract class DefaultJavaLoweringProvider implements LoweringProvider {
         JavaKind elementKind = storeIndexed.elementKind();
 
         LogicNode condition = null;
-        if (elementKind == JavaKind.Object && !StampTool.isPointerAlwaysNull(value)) {
+        if (storeIndexed.getStoreCheck() == null && elementKind == JavaKind.Object && !StampTool.isPointerAlwaysNull(value)) {
             /* Array store check. */
             TypeReference arrayType = StampTool.typeReferenceOrNull(array);
             if (arrayType != null && arrayType.isExact()) {
@@ -509,6 +525,12 @@ public abstract class DefaultJavaLoweringProvider implements LoweringProvider {
         loadHub.replaceAtUsagesAndDelete(hub);
     }
 
+    protected void lowerLoadArrayComponentHubNode(LoadArrayComponentHubNode loadHub) {
+        StructuredGraph graph = loadHub.graph();
+        ValueNode hub = createReadArrayComponentHub(graph, loadHub.getValue(), loadHub);
+        graph.replaceFixed(loadHub, hub);
+    }
+
     protected void lowerMonitorEnterNode(MonitorEnterNode monitorEnter, LoweringTool tool, StructuredGraph graph) {
         ValueNode object = createNullCheckedValue(monitorEnter.object(), monitorEnter, tool);
         ValueNode hub = graph.addOrUnique(LoadHubNode.create(object, tool.getStampProvider(), tool.getMetaAccess(), tool.getConstantReflection()));
@@ -526,11 +548,28 @@ public abstract class DefaultJavaLoweringProvider implements LoweringProvider {
         ValueNode newValue = implicitStoreConvert(graph, valueKind, cas.newValue());
 
         AddressNode address = graph.unique(new OffsetAddressNode(cas.object(), cas.offset()));
-        LogicCompareAndSwapNode atomicNode = graph.add(new LogicCompareAndSwapNode(address, cas.getLocationIdentity(), expectedValue, newValue, compareAndSwapBarrierType(cas)));
+        BarrierType barrierType = guessStoreBarrierType(cas.object(), expectedValue);
+        LogicCompareAndSwapNode atomicNode = graph.add(new LogicCompareAndSwapNode(address, cas.getLocationIdentity(), expectedValue, newValue, barrierType));
         atomicNode.setStateAfter(cas.stateAfter());
         graph.replaceFixedWithFixed(cas, atomicNode);
     }
 
+    protected void lowerCompareAndExchangeNode(UnsafeCompareAndExchangeNode cas) {
+        StructuredGraph graph = cas.graph();
+        JavaKind valueKind = cas.getValueKind();
+
+        ValueNode expectedValue = implicitStoreConvert(graph, valueKind, cas.expected());
+        ValueNode newValue = implicitStoreConvert(graph, valueKind, cas.newValue());
+
+        AddressNode address = graph.unique(new OffsetAddressNode(cas.object(), cas.offset()));
+        BarrierType barrierType = guessStoreBarrierType(cas.object(), expectedValue);
+        ValueCompareAndSwapNode atomicNode = graph.add(new ValueCompareAndSwapNode(address, expectedValue, newValue, cas.getLocationIdentity(), barrierType));
+        ValueNode coercedNode = implicitLoadConvert(graph, valueKind, atomicNode, true);
+        atomicNode.setStateAfter(cas.stateAfter());
+        cas.replaceAtUsages(coercedNode);
+        graph.replaceFixedWithFixed(cas, atomicNode);
+    }
+
     protected void lowerAtomicReadAndWriteNode(AtomicReadAndWriteNode n) {
         StructuredGraph graph = n.graph();
         JavaKind valueKind = n.getValueKind();
@@ -538,7 +577,9 @@ public abstract class DefaultJavaLoweringProvider implements LoweringProvider {
         ValueNode newValue = implicitStoreConvert(graph, valueKind, n.newValue());
 
         AddressNode address = graph.unique(new OffsetAddressNode(n.object(), n.offset()));
-        LoweredAtomicReadAndWriteNode memoryRead = graph.add(new LoweredAtomicReadAndWriteNode(address, n.getLocationIdentity(), newValue, atomicReadAndWriteBarrierType(n)));
+        BarrierType barrierType = guessStoreBarrierType(n.object(), n.newValue());
+        LIRKind lirAccessKind = LIRKind.fromJavaKind(target.arch, valueKind);
+        LoweredAtomicReadAndWriteNode memoryRead = graph.add(new LoweredAtomicReadAndWriteNode(address, n.getLocationIdentity(), newValue, lirAccessKind, barrierType));
         memoryRead.setStateAfter(n.stateAfter());
 
         ValueNode readValue = implicitLoadConvert(graph, valueKind, memoryRead);
@@ -673,6 +714,7 @@ public abstract class DefaultJavaLoweringProvider implements LoweringProvider {
         memoryWrite.setGuard(write.getGuard());
     }
 
+    @SuppressWarnings("try")
     protected void lowerCommitAllocationNode(CommitAllocationNode commit, LoweringTool tool) {
         StructuredGraph graph = commit.graph();
         if (graph.getGuardsStage() == StructuredGraph.GuardsStage.FIXED_DEOPTS) {
@@ -683,83 +725,87 @@ public abstract class DefaultJavaLoweringProvider implements LoweringProvider {
             int valuePos = 0;
             for (int objIndex = 0; objIndex < commit.getVirtualObjects().size(); objIndex++) {
                 VirtualObjectNode virtual = commit.getVirtualObjects().get(objIndex);
-                int entryCount = virtual.entryCount();
-                AbstractNewObjectNode newObject;
-                if (virtual instanceof VirtualInstanceNode) {
-                    newObject = graph.add(createNewInstanceFromVirtual(virtual));
-                } else {
-                    newObject = graph.add(createNewArrayFromVirtual(virtual, ConstantNode.forInt(entryCount, graph)));
-                }
-                recursiveLowerings.add(newObject);
-                graph.addBeforeFixed(commit, newObject);
-                allocations[objIndex] = newObject;
-                for (int i = 0; i < entryCount; i++) {
-                    ValueNode value = commit.getValues().get(valuePos);
-                    if (value instanceof VirtualObjectNode) {
-                        value = allocations[commit.getVirtualObjects().indexOf(value)];
+                try (DebugCloseable nsp = graph.withNodeSourcePosition(virtual)) {
+                    int entryCount = virtual.entryCount();
+                    AbstractNewObjectNode newObject;
+                    if (virtual instanceof VirtualInstanceNode) {
+                        newObject = graph.add(createNewInstanceFromVirtual(virtual));
+                    } else {
+                        newObject = graph.add(createNewArrayFromVirtual(virtual, ConstantNode.forInt(entryCount, graph)));
                     }
-                    if (value == null) {
-                        omittedValues.set(valuePos);
-                    } else if (!(value.isConstant() && value.asConstant().isDefaultForKind())) {
-                        // Constant.illegal is always the defaultForKind, so it is skipped
-                        JavaKind valueKind = value.getStackKind();
-                        JavaKind entryKind = virtual.entryKind(i);
 
-                        // Truffle requires some leniency in terms of what can be put where:
-                        assert valueKind.getStackKind() == entryKind.getStackKind() ||
-                                        (valueKind == JavaKind.Long || valueKind == JavaKind.Double || (valueKind == JavaKind.Int && virtual instanceof VirtualArrayNode));
-                        AddressNode address = null;
-                        BarrierType barrierType = null;
-                        if (virtual instanceof VirtualInstanceNode) {
-                            ResolvedJavaField field = ((VirtualInstanceNode) virtual).field(i);
-                            long offset = fieldOffset(field);
-                            if (offset >= 0) {
-                                address = createOffsetAddress(graph, newObject, offset);
-                                barrierType = fieldInitializationBarrier(entryKind);
+                    recursiveLowerings.add(newObject);
+                    graph.addBeforeFixed(commit, newObject);
+                    allocations[objIndex] = newObject;
+                    for (int i = 0; i < entryCount; i++) {
+                        ValueNode value = commit.getValues().get(valuePos);
+                        if (value instanceof VirtualObjectNode) {
+                            value = allocations[commit.getVirtualObjects().indexOf(value)];
+                        }
+                        if (value == null) {
+                            omittedValues.set(valuePos);
+                        } else if (!(value.isConstant() && value.asConstant().isDefaultForKind())) {
+                            // Constant.illegal is always the defaultForKind, so it is skipped
+                            JavaKind valueKind = value.getStackKind();
+                            JavaKind entryKind = virtual.entryKind(i);
+
+                            // Truffle requires some leniency in terms of what can be put where:
+                            assert valueKind.getStackKind() == entryKind.getStackKind() ||
+                                            (valueKind == JavaKind.Long || valueKind == JavaKind.Double || (valueKind == JavaKind.Int && virtual instanceof VirtualArrayNode));
+                            AddressNode address = null;
+                            BarrierType barrierType = null;
+                            if (virtual instanceof VirtualInstanceNode) {
+                                ResolvedJavaField field = ((VirtualInstanceNode) virtual).field(i);
+                                long offset = fieldOffset(field);
+                                if (offset >= 0) {
+                                    address = createOffsetAddress(graph, newObject, offset);
+                                    barrierType = fieldInitializationBarrier(entryKind);
+                                }
+                            } else {
+                                address = createOffsetAddress(graph, newObject, tool.getMetaAccess().getArrayBaseOffset(entryKind) + i * arrayScalingFactor(entryKind));
+                                barrierType = arrayInitializationBarrier(entryKind);
+                            }
+                            if (address != null) {
+                                WriteNode write = new WriteNode(address, LocationIdentity.init(), implicitStoreConvert(graph, entryKind, value), barrierType);
+                                graph.addAfterFixed(newObject, graph.add(write));
                             }
-                        } else {
-                            address = createOffsetAddress(graph, newObject, arrayBaseOffset(entryKind) + i * arrayScalingFactor(entryKind));
-                            barrierType = arrayInitializationBarrier(entryKind);
-                        }
-                        if (address != null) {
-                            WriteNode write = new WriteNode(address, LocationIdentity.init(), implicitStoreConvert(graph, entryKind, value), barrierType);
-                            graph.addAfterFixed(newObject, graph.add(write));
                         }
+                        valuePos++;
                     }
-                    valuePos++;
-
                 }
             }
             valuePos = 0;
 
             for (int objIndex = 0; objIndex < commit.getVirtualObjects().size(); objIndex++) {
                 VirtualObjectNode virtual = commit.getVirtualObjects().get(objIndex);
-                int entryCount = virtual.entryCount();
-                ValueNode newObject = allocations[objIndex];
-                for (int i = 0; i < entryCount; i++) {
-                    if (omittedValues.get(valuePos)) {
-                        ValueNode value = commit.getValues().get(valuePos);
-                        assert value instanceof VirtualObjectNode;
-                        ValueNode allocValue = allocations[commit.getVirtualObjects().indexOf(value)];
-                        if (!(allocValue.isConstant() && allocValue.asConstant().isDefaultForKind())) {
-                            assert virtual.entryKind(i) == JavaKind.Object && allocValue.getStackKind() == JavaKind.Object;
-                            AddressNode address;
-                            BarrierType barrierType;
-                            if (virtual instanceof VirtualInstanceNode) {
-                                VirtualInstanceNode virtualInstance = (VirtualInstanceNode) virtual;
-                                address = createFieldAddress(graph, newObject, virtualInstance.field(i));
-                                barrierType = BarrierType.IMPRECISE;
-                            } else {
-                                address = createArrayAddress(graph, newObject, virtual.entryKind(i), ConstantNode.forInt(i, graph));
-                                barrierType = BarrierType.PRECISE;
-                            }
-                            if (address != null) {
-                                WriteNode write = new WriteNode(address, LocationIdentity.init(), implicitStoreConvert(graph, JavaKind.Object, allocValue), barrierType);
-                                graph.addBeforeFixed(commit, graph.add(write));
+                try (DebugCloseable nsp = graph.withNodeSourcePosition(virtual)) {
+                    int entryCount = virtual.entryCount();
+                    ValueNode newObject = allocations[objIndex];
+                    for (int i = 0; i < entryCount; i++) {
+                        if (omittedValues.get(valuePos)) {
+                            ValueNode value = commit.getValues().get(valuePos);
+                            assert value instanceof VirtualObjectNode;
+                            ValueNode allocValue = allocations[commit.getVirtualObjects().indexOf(value)];
+                            if (!(allocValue.isConstant() && allocValue.asConstant().isDefaultForKind())) {
+                                assert virtual.entryKind(i) == JavaKind.Object && allocValue.getStackKind() == JavaKind.Object;
+                                AddressNode address;
+                                BarrierType barrierType;
+                                if (virtual instanceof VirtualInstanceNode) {
+                                    VirtualInstanceNode virtualInstance = (VirtualInstanceNode) virtual;
+                                    address = createFieldAddress(graph, newObject, virtualInstance.field(i));
+                                    barrierType = BarrierType.IMPRECISE;
+                                } else {
+                                    address = createArrayAddress(graph, newObject, virtual.entryKind(i), ConstantNode.forInt(i, graph));
+                                    barrierType = BarrierType.PRECISE;
+                                }
+                                if (address != null) {
+                                    WriteNode write = new WriteNode(address, LocationIdentity.init(), implicitStoreConvert(graph, JavaKind.Object, allocValue), barrierType);
+                                    graph.addBeforeFixed(commit, graph.add(write));
+                                }
                             }
                         }
+                        valuePos++;
                     }
-                    valuePos++;
                 }
             }
 
@@ -770,6 +816,7 @@ public abstract class DefaultJavaLoweringProvider implements LoweringProvider {
                 recursiveLowering.lower(tool);
             }
         }
+
     }
 
     public NewInstanceNode createNewInstanceFromVirtual(VirtualObjectNode virtual) {
@@ -884,28 +931,22 @@ public abstract class DefaultJavaLoweringProvider implements LoweringProvider {
         return entryKind == JavaKind.Object ? BarrierType.PRECISE : BarrierType.NONE;
     }
 
-    protected BarrierType unsafeStoreBarrierType(RawStoreNode store) {
+    private BarrierType unsafeStoreBarrierType(RawStoreNode store) {
         if (!store.needsBarrier()) {
             return BarrierType.NONE;
         }
-        return storeBarrierType(store.object(), store.value());
+        return guessStoreBarrierType(store.object(), store.value());
     }
 
-    protected BarrierType compareAndSwapBarrierType(UnsafeCompareAndSwapNode cas) {
-        return storeBarrierType(cas.object(), cas.expected());
-    }
-
-    protected BarrierType atomicReadAndWriteBarrierType(AtomicReadAndWriteNode n) {
-        return storeBarrierType(n.object(), n.newValue());
-    }
-
-    protected BarrierType storeBarrierType(ValueNode object, ValueNode value) {
-        if (value.getStackKind() == JavaKind.Object) {
+    private BarrierType guessStoreBarrierType(ValueNode object, ValueNode value) {
+        if (value.getStackKind() == JavaKind.Object && object.getStackKind() == JavaKind.Object) {
             ResolvedJavaType type = StampTool.typeOrNull(object);
-            if (type != null && !type.isArray()) {
-                return BarrierType.IMPRECISE;
-            } else {
+            // Array types must use a precise barrier, so if the type is unknown or is a supertype
+            // of Object[] then treat it as an array.
+            if (type == null || type.isArray() || type.isAssignableFrom(objectArrayType)) {
                 return BarrierType.PRECISE;
+            } else {
+                return BarrierType.IMPRECISE;
             }
         }
         return BarrierType.NONE;
@@ -921,8 +962,7 @@ public abstract class DefaultJavaLoweringProvider implements LoweringProvider {
 
     public abstract int arrayLengthOffset();
 
-    public abstract int arrayBaseOffset(JavaKind elementKind);
-
+    @Override
     public int arrayScalingFactor(JavaKind elementKind) {
         return target.arch.getPlatformKind(elementKind).getSizeInBytes();
     }
@@ -1033,6 +1073,10 @@ public abstract class DefaultJavaLoweringProvider implements LoweringProvider {
     protected abstract ValueNode createReadArrayComponentHub(StructuredGraph graph, ValueNode arrayHub, FixedNode anchor);
 
     protected GuardingNode getBoundsCheck(AccessIndexedNode n, ValueNode array, LoweringTool tool) {
+        if (n.getBoundsCheck() != null) {
+            return n.getBoundsCheck();
+        }
+
         StructuredGraph graph = n.graph();
         ValueNode arrayLength = readArrayLength(array, tool.getConstantReflection());
         if (arrayLength == null) {
@@ -1052,7 +1096,7 @@ public abstract class DefaultJavaLoweringProvider implements LoweringProvider {
         if (StampTool.isPointerNonNull(object)) {
             return null;
         }
-        return tool.createGuard(before, before.graph().unique(IsNullNode.create(object)), NullCheckException, InvalidateReprofile, JavaConstant.NULL_POINTER, true);
+        return tool.createGuard(before, before.graph().unique(IsNullNode.create(object)), NullCheckException, InvalidateReprofile, SpeculationLog.NO_SPECULATION, true, null);
     }
 
     protected ValueNode createNullCheckedValue(ValueNode object, FixedNode before, LoweringTool tool) {
